#
# Copyright 2015 Cloudbase Solutions SRL
#

from charmhelpers.core.hookenv import (
    config,
    log,
    relation_set,
)
from charmhelpers.core.host import (
    restart_on_change,
)
from charmhelpers.fetch import (
    apt_install,
    apt_update,
)
from charmhelpers.contrib.openstack.utils import (
    configure_installation_source,
    openstack_upgrade_available,
)
from nova_ironic_utils import (
    register_configs,
    cmd_all_services,
    restart_map,
    do_openstack_upgrade,
)
import constants

CONFIGS = register_configs()


def install():
    configure_installation_source(config('openstack-origin'))
    apt_update()
    apt_install(constants.PACKAGES, fatal=True)


@restart_on_change(restart_map())
def config_changed():
    if openstack_upgrade_available('nova-common'):
        do_openstack_upgrade(CONFIGS)
    CONFIGS.write_all()


def start():
    cmd_all_services('start')


def stop():
    cmd_all_services('stop')


def amqp_joined(relation_id=None):
    relation_set(relation_id=relation_id,
                 username=config('rabbit-user'),
                 vhost=config('rabbit-vhost'))


@restart_on_change(restart_map())
def amqp_changed():
    if 'amqp' not in CONFIGS.complete_contexts():
        log('amqp relation incomplete. Peer not ready?')
        return
    CONFIGS.write_all()


@restart_on_change(restart_map())
def ironic_api_changed(relation_id=None):
    if 'ironic-api' not in CONFIGS.complete_contexts():
        log('ironic-api relation incomplete. Peer not ready?')
        return
    CONFIGS.write_all()


@restart_on_change(restart_map())
def cloud_compute_changed():
    CONFIGS.write_all()
